import random
import time
import uuid

from .exceptions import GenshinHelperException
from .utils import log, request, get_ds, extract_cookie, _


TOKEN_URL = 'https://api-takumi.mihoyo.com/auth/api/getMultiTokenByLoginTicket?login_ticket={}&token_types=3&uid={}'


def get_app_cookie(cookie):
    if 'stoken' in cookie:
        return cookie

    login_ticket = extract_cookie('login_ticket', cookie)
    stuid = extract_cookie('account_id', cookie)
    url = TOKEN_URL.format(login_ticket, stuid)
    response = request('get', url).json()
    if response.get('message') != 'OK':
        raise GenshinHelperException('Failed to get multi token: '
            'The cookie seems to be invalid, please re-login to bbs.mihoyo.com')

    stoken = response['data']['list'][0]['token']
    app_cookie = f'stuid={stuid}; stoken={stoken}; login_ticket={login_ticket}'
    raise GenshinHelperException(f'YOUR COOKIE_MIYOUBI:\n{app_cookie}')


class MiyoubiCheckin(object):
    STATE_URL = 'https://bbs-api.mihoyo.com/apihub/sapi/getUserMissionsState'
    SIGN_URL = 'https://bbs-api.mihoyo.com/apihub/sapi/signIn?gids={}'
    POST_LIST_URL = 'https://bbs-api.mihoyo.com/post/api/getForumPostList?forum_id={}&is_good=false&is_hot=false&page_size=20&sort_type=1'
    POST_FULL_URL = 'https://bbs-api.mihoyo.com/post/api/getPostFull?post_id={}'
    UPVOTE_URL = 'https://bbs-api.mihoyo.com/apihub/sapi/upvotePost'
    SHARE_URL = 'https://bbs-api.mihoyo.com/apihub/api/getShareConf?entity_id={}&entity_type=1'

    def __init__(self, cookie: str):
        self._cookie = get_app_cookie(cookie)
        self.get_missions_state()

    def get_header(self):
        client_type, app_version, ds = get_ds('android')
        header = {
            'Cookie': self._cookie,
            'User-Agent': 'okhttp/4.8.0',
            'Referer': 'https://app.mihoyo.com',
            'Accept-Encoding': 'gzip, deflate, br',
            'x-rpc-channel': 'miyousheluodi',
            'x-rpc-device_id': str(uuid.uuid3(uuid.NAMESPACE_URL, self._cookie)).replace(
                '-', '').upper(),
            'x-rpc-client_type': client_type,
            'x-rpc-app_version': app_version,
            'DS': ds
        }
        return header

    def get_missions_state(self):
        url = self.STATE_URL
        response = request('get', url, headers=self.get_header()).json()
        self.total_points = response.get('data', {}).get('total_points', -1)
        states = response.get('data', {}).get('states', [])
        self.missions_state = {
            i['mission_key']: i['is_get_award'] for i in states if i['mission_id'] in (58, 59, 60, 61)
        }
        self.is_sign = self.missions_state.get('continuous_sign', False)
        self.is_view = self.missions_state.get('view_post_0', False)
        self.is_upvote = self.missions_state.get('post_up_0', False)
        self.is_share = self.missions_state.get('share_post_0', False)

    def sign(self, id: int):
        forums = {1: '崩坏3', 2: '原神', 3: '崩坏2', 4: '未定事件簿', 5: '大别野'}
        if id not in range(1, 6):
            raise ValueError('The value of id is one of 1, 2, 3, 4, 5')

        url = self.SIGN_URL.format(id)
        response = request('post', url, headers=self.get_header()).json()
        message = response.get('message')
        result = (False, message)
        if message == 'OK' or '重复' in message:
            result = (True, message)

        log.info(f'{forums[id]}: {message}')
        return result

    def get_posts(self, forum_id: int):
        # 1: 崩坏3  26: 原神  30: 崩坏2  37: 未定事件簿  34: 大别野
        forum_ids = (1, 26, 30, 37, 34)
        if forum_id not in forum_ids:
            raise ValueError('The value of forum_id is one of 1, 26, 30, 37, 34')

        url = self.POST_LIST_URL.format(forum_id)
        response = request('get', url).json()
        message = response.get('message')
        if message != 'OK':
            raise GenshinHelperException(f'Failed to get posts:\n{message}')

        post_list = response.get('data', {}).get('list', [])
        posts = [{
            'post_id': post.get('post', {}).get('post_id', 0),
            'title': post.get('post', {}).get('subject', 'untitled')
        } for post in post_list if post_list]

        log.info(_('Succeeded in getting {num_posts} posts').format(
            num_posts=len(posts)
            ))
        return posts

    def view_post(self, post: dict):
        time.sleep(3)
        post_id = post.get('post_id')
        title = post.get('title')
        url = self.POST_FULL_URL.format(post_id)
        response = request('get', url, headers=self.get_header()).json()
        message = response.get('message')
        result = (False, title)
        if message == 'OK':
            result = (True, title)

        log.info(f'{title}: {message}')
        return result

    def upvote_post(self, post: dict):
        time.sleep(3)
        post_id = post.get('post_id')
        title = post.get('title')
        url = self.UPVOTE_URL
        data = {'post_id': post_id, 'is_cancel': False}
        response = request('post', url, json=data, headers=self.get_header()).json()
        message = response.get('message')
        result = (False, title)
        if message == 'OK':
            result = (True, title)

        log.info(f'{title}: {message}')
        return result

    def share_post(self, post: dict):
        post_id = post.get('post_id')
        title = post.get('title')
        url = self.SHARE_URL.format(post_id)
        response = request('get', url, headers=self.get_header()).json()
        message = response.get('message')
        result = (False, title)
        if message == 'OK':
            result = (True, title)

        log.info(f'{title}: {message}')
        return result

    def run(self):
        def get_result(name, info):
            return f'{name} ({[i[0] for i in info].count(True)}/{len(info)})'

        log.info(f'🌕 {self.total_points}')
        log.info(_('Preparing to get posts...'))
        forum_id = 26
        posts = self.get_posts(forum_id)

        log.info(_('Preparing to check-in...'))
        sign = [self.sign(i) for i in range(1, 6) if not self.is_sign]
        log.info(_('Preparing to view posts...'))
        view = [self.view_post(i) for i in random.sample(posts[0:5], 3) if not self.is_view]
        log.info(_('Preparing to upvote posts...'))
        upvote = [self.upvote_post(i) for i in random.sample(posts[5:17], 10) if not self.is_upvote]
        log.info(_('Preparing to share post...'))
        share = [self.share_post(i) for i in random.sample(posts[-3:-1], 1) if not self.is_share]

        message_box = [
            get_result('签到', sign),
            get_result('浏览', view),
            get_result('点赞', upvote),
            get_result('分享', share)
        ]

        return '\n    '.join(message_box)
      



import json
import time
import uuid

from .utils import MESSAGE_TEMPLATE, log, request, _


class Bh3Checkin(object):
    ACT_ID = 'e202104072769'
    USER_AGENT = 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) miHoYoBBS/2.8.0'
    REFERER_URL = 'https://webstatic.mihoyo.com/bh3/event/euthenia/index.html?bbs_presentation_style={}&bbs_game_role_required={}&bbs_auth_required={}&act_id={}&utm_source={}&utm_medium={}&utm_campaign={}'.format(
        'fullscreen', 'bh3_cn', 'true', ACT_ID, 'bbs', 'mys', 'icon')
    ROLES_INFO = {}
    INFO_URL = ''
    ROLE_URL = 'https://api-takumi.mihoyo.com/binding/api/getUserGameRolesByCookie?game_biz={}'.format(
        'bh3_cn')
    _INFO_URL = 'https://api-takumi.mihoyo.com/common/euthenia/index?region={}&act_id={}&uid={}'
    SIGN_URL = 'https://api-takumi.mihoyo.com/common/euthenia/sign'

    def __init__(self, cookie: str=None):
        self._cookie = cookie
        self._sign_data = {}

    def get_header(self):
        header = {
            'Cookie':
            self._cookie,
            'User-Agent':
            self.USER_AGENT,
            'Referer':
            self.REFERER_URL,
            'Accept-Encoding':
            'gzip, deflate, br',
            'x-rpc-device_id':
            str(uuid.uuid3(uuid.NAMESPACE_URL, self._cookie)).replace(
                '-', '').upper()
        }
        return header

    def get_weekly_finance(self, bind_uid, bind_region):
        # level > 25
        url = 'https://api.mihoyo.com/bh3-weekly_finance/api/index'
        payload = {
            'game_biz': 'bh3_cn',
            'bind_uid': bind_uid,
            'bind_region': bind_region
        }
        response = request(
            'get', url, headers=self.get_header(), params=payload).json()

        return response

    @property
    def roles_list(self):
        response = request(
            'get', self.ROLE_URL, headers=self.get_header()).json()

        if response.get('retcode', 1) != 0:
            raise Exception(response.get('message', 'Empty roles data'))

        roles_list = response.get('data', {}).get('list', [])
        return roles_list

    @property
    def _sign_info(self):
        response = request(
            'get', self.INFO_URL, headers=self.get_header()).json()
        return response

    def sign(self):
        sign_info = self._sign_info.get('data', {}).get('sign', {})

        today = sign_info.get('now', '1970-01-01 00:00:00').split(' ')[0]
        awards = sign_info.get('list', [])
        # 0: can not check in, 1: can check in, 2: already checked in
        current_day = len(awards) - [i['status'] for i in awards].count(0)
        total_sign_day = [i['status'] for i in awards].count(2)
        is_sign = True if awards[current_day - 1]['status'] == 2 else False
        region_name = self._sign_data.get('region_name', 'Global')
        uid = self._sign_data.get('uid', 123456789)
        hidden_uid = str(uid).replace(str(uid)[3:-3], '***', 1)
        data = self._sign_data.get('post_data', {})
        region = self._sign_data.get('region')

        travel_notes = '希望不熄 薪火永燃'
        weekly_finance = self.get_weekly_finance(uid, region)
        if weekly_finance.get('retcode', -9999) == 0:
            finance = weekly_finance.get('data', {})
            travel_notes = _('''舰长的 {} 月手帐
    💎水晶: {}
    🔮星石: {}''').format(finance['month'], finance['month_hcoin'],
                       finance['month_star'])

        message = {
            'today': today,
            'region_name': region_name,
            'uid': hidden_uid,
            'total_sign_day': total_sign_day,
            'travel_notes': travel_notes,
            'end': ''
        }

        log.info(_('准备为舰长 {hidden_uid} 签到...').format(hidden_uid=hidden_uid))
        time.sleep(5)

        if is_sign:
            message['award_name'] = awards[total_sign_day - 1].get('name')
            message['award_cnt'] = awards[total_sign_day - 1].get('cnt')
            message['status'] = _('👀 舰长, 你已经签到过了哦')

            return self.message.format(**message)
        else:
            message['award_name'] = awards[total_sign_day].get('name')
            message['award_cnt'] = awards[total_sign_day].get('cnt')

        try:
            response = request(
                'post',
                self.SIGN_URL,
                headers=self.get_header(),
                data=json.dumps(data, ensure_ascii=False)).json()
        except Exception as e:
            raise Exception(e)
        # 0:      success
        # -5003:  already checked in
        code = response.get('retcode', 99999)
        if code != 0:
            return response
        message['total_sign_day'] = total_sign_day + 1
        message['status'] = response.get('message')

        log.info(_('签到完毕'))
        return self.message.format(**message)

    def run(self):
        roles_list = self.roles_list
        if roles_list:
            result_list = []
            log.info(
                _('当前账号绑定了 {num_roles} 个角色').format(num_roles=len(roles_list)))
            for role in roles_list:

                region = role.get('region', 'cn')
                region_name = role.get('region_name', 'CN')
                uid = role.get('game_uid', 123456789)
                self.INFO_URL = self._INFO_URL.format(region, self.ACT_ID, uid)
                post_data = {
                    'act_id': self.ACT_ID,
                    'region': region,
                    'uid': uid
                }
                self._sign_data.update({
                    'region': region,
                    'region_name': region_name,
                    'uid': uid,
                    'post_data': post_data
                })
                result_list.append(self.sign())
            return ''.join(result_list)

    @property
    def message(self):
        return MESSAGE_TEMPLATE.replace('月', '周', 1)




import base64
import hashlib
import hmac
import json
import time
from urllib import parse

from .basenotifier import BaseNotifier as Base
from ..config import config


class DingTalkBot(Base):
    def __init__(self):
        self.name = 'DingTalk Bot'
        self.token = config.DD_BOT_TOKEN
        self.retcode_key = 'errcode'
        self.retcode_value = 0

    # Fixed by wananing
    def send(self, text=Base.app, status=Base.status, desp=Base.desp):
        url = ''
        if config.DD_BOT_TOKEN:
            url = f'https://oapi.dingtalk.com/robot/send?access_token={config.DD_BOT_TOKEN}'
            if config.DD_BOT_SECRET:
                secret = config.DD_BOT_SECRET
                timestamp = int(round(time.time() * 1000))
                secret_enc = secret.encode('utf-8')
                string_to_sign = f'{timestamp}\n{secret}'
                string_to_sign_enc = string_to_sign.encode('utf-8')
                hmac_code = hmac.new(
                    secret_enc, string_to_sign_enc,
                    digestmod=hashlib.sha256).digest()
                sign = parse.quote_plus(base64.b64encode(hmac_code))
                url = f'https://oapi.dingtalk.com/robot/send?access_token={config.DD_BOT_TOKEN}&timestamp={timestamp}&sign={sign}'

        header = {'Content-Type': 'application/json ;charset=utf-8 '}
        data = {
            'msgtype': 'text',
            'text': {
                'content': f'{text} {status}\n\n{desp}'
            }
        }
        data = json.dumps(data)
        return self.push('post', url, data=data, headers=header)



import re
import requests

from bs4 import BeautifulSoup

from . import notifiers
from .config import config
from .exceptions import GenshinHelperException
from .utils import log, request, extract_cookie


class SuperTopicCheckin(object):
    USER_AGENT = 'WeiboOverseas/4.1.4 (iPhone; iOS 14.0.1; Scale/2.00)'
    WEIBO_API = 'https://api.weibo.cn'
    FOLLOW_DATA_URL = WEIBO_API + '/2/cardlist'
    SIGN_URL = WEIBO_API + '/2/page/button'

    def __init__(self, cookie: str = None):
        self._cookie = cookie
        self._cookie_wb = config.COOKIE_KA
        self.headers = {
            'User-Agent': self.USER_AGENT,
            'Cookie': self._cookie_wb
        }

    def get_follow_data(self):
        url = self.FOLLOW_DATA_URL
        data = {
            'containerid': '100803_-_followsuper',
            'c': 'weicoabroad',
            's': extract_cookie('s', self._cookie),
            'gsid': extract_cookie('gsid', self._cookie),
            'aid': extract_cookie('aid', self._cookie),   # ios
            'from': extract_cookie('from', self._cookie)  # android
        }
        # turn off certificate verification
        response = request('get', url, params=data, headers=self.headers, verify=False).json()
        errno = response.get('errno')
        if errno:
            raise GenshinHelperException(f'Failed to get follow super:\n{response}')

        card_group = response.get('cards', [{}])[0].get('card_group', [])
        follow_list = [
            card for card in card_group if card.get('card_type') == '8'
        ]
        if not follow_list:
            raise GenshinHelperException('Failed to get follow list, '
            'please make sure you have followed the Genshin Impact super topic.')

        follow_data = []
        for item in follow_list:
            action = item.get('buttons', [{}])[0].get('params', {}).get(
                'action', '')
            request_url = ''.join(
                re.findall('(?<=request_url=)(.*)%26container', action))
            sign_data = {k: v for (k, v) in data.items()}
            del sign_data['containerid']
            sign_data['request_url'] = request_url

            follow = {
                'name': item.get('title_sub'),
                'lv': int(re.findall('\d+', item.get('desc1', '0'))[0]),
                'is_sign': item.get('buttons', [{}])[0].get('name'),
                'sign_data': sign_data
            }
            follow_data.append(follow)

        follow_data.sort(key=lambda k: (k.get('lv', 0)), reverse=True)

        return follow_data

    def run(self):
        follow_data = self.get_follow_data()
        message_box = []
        for follow in follow_data:
            name = follow.get('name')
            lv = follow.get('lv')
            is_sign = follow.get('is_sign')
            data = follow.get('sign_data', {})
            name_style = f'⚜️ [Lv.{lv}]{name}'

            # Already checked in
            if is_sign == '已签':
                message_box.append(f'{name_style} ☑️')
            elif is_sign == '签到':
                url = self.SIGN_URL
                # turn off certificate verification
                response = request('get', url, params=data, headers=self.headers, verify=False).json()
                if response.get('result') in (1, '1'):
                    message_box.append(f'{name_style} ✅')
                # Already checked in
                elif '已签到' in response.get('msg', ''):
                    message_box.append(f'{name_style} ☑️')
                else:
                    raise GenshinHelperException(f'{name_style} :Failed to run sign:\n{response}')
            else:
                raise GenshinHelperException('Unknown check-in status')

        return '\n    '.join(message_box)


class RedemptionCode(object):
    USER_AGENT = 'WeiboOverseas/4.1.4 (iPhone; iOS 14.0.1; Scale/2.00)'
    CONTAINER_ID = '100808fc439dedbb06ca5fd858848e521b8716'
    GENSHIN_CHAOHUA_URL = 'https://m.weibo.cn/api/container/getIndex?containerid={}_-_feed'.format(CONTAINER_ID)
    DRAW_URL = 'https://ka.sina.com.cn/innerapi/draw'
    MYBOX_URL = 'https://ka.sina.com.cn/html5/mybox'

    def __init__(self, cookie: str = None):
        self._cookie = self.get_ka_cookie(cookie)
        self.headers = {
            'User-Agent': self.USER_AGENT,
            'Referer': 'https://ka.sina.com.cn/html5'
        }

    @staticmethod
    def get_ka_cookie(cookie):
        SUB = extract_cookie('SUB', cookie)
        SUBP = extract_cookie('SUBP', cookie)

        session = requests.session()
        session.get('https://ka.sina.com.cn/html5')
        jar = requests.cookies.RequestsCookieJar()
        jar.set('SUB', SUB)
        jar.set('SUBP', SUBP)
        session.cookies.update(jar)
        cookie_ka = session.cookies.get_dict()
        return cookie_ka

    @property
    def event_gift_ids(self):
        is_event = self.check_event()
        if not is_event:
            return []

        group = self.event_card[0].get('card_group', [{}])[0].get('group', [])
        ids = [i for item in group if '签到' in item.get('title_sub') for i in re.findall(
            r'gift/(\d*)', item['scheme'])]
        return ids

    @property
    def mybox_codes(self):
        url = self.MYBOX_URL
        response = request(
            'get', url, headers=self.headers, cookies=self._cookie, allow_redirects=False)
        if response.status_code != 200:
            raise GenshinHelperException('Failed to get my box codes: '
            'The cookie seems to be invalid, please re-login to m.weibo.cn')

        response.encoding = 'utf-8'
        soup = BeautifulSoup(response.text, 'html.parser')
        # print(soup.prettify())
        boxs = soup.find_all(class_='giftbag')
        mybox_codes = []
        for box in boxs:
            item = {
                'id': box.find(class_='deleBtn').get('data-itemid'),
                'title': box.find(class_='title itemTitle').text,
                'code': box.find('span').parent.contents[1]
            }
            mybox_codes.append(item)

        return mybox_codes

    @property
    def mybox_gift_ids(self):
        mybox_codes = self.gift_bag = self.mybox_codes
        ids = [item.get('id') for item in mybox_codes]
        return ids

    @property
    def unclaimed_gift_ids(self):
        event_gift_ids = self.event_gift_ids
        mybox_gift_ids = self.mybox_gift_ids
        ids = [i for i in event_gift_ids if i not in mybox_gift_ids]
        return ids

    def check_event(self, subscribe = False):
        url = self.GENSHIN_CHAOHUA_URL
        response = request('get', url, headers=self.headers).json()
        cards = response.get('data', {}).get('cards', [])
        event_card = [
            card for card in cards if card.get('itemid') == 'pagemanual_2'
        ]
        if not event_card:
            return False

        if subscribe:
            message = (
                '原神超话签到提醒',
                '亲爱的旅行者，原神微博超话签到活动现已开启，请注意活动时间！如已完成任务，请忽略本信息。'
            )
            notifiers.send2all(status=message[0], desp=message[1])

        self.event_card = event_card
        return True

    def get_code(self, id):
        headers = self.headers
        headers['Referer'] = f'https://ka.sina.com.cn/html5/gift/{id}'
        url = self.DRAW_URL
        data = {'gid': 10725, 'itemId': id, 'channel': 'wblink'}
        response = request('get', url, params=data, headers=headers, cookies=self._cookie).json()

        if response.get('k'):
            log.info(f'The redemption code of ID {id} received successfully')
            return response['data']['kahao']
        elif response.get('code') == '2002' and '头像' in response.get('msg', ''):
            log.error(f'{id}: You can only receive one redemption code per day '
            'or the code has already been received!')
        elif response.get('code') == '2002':
            log.error(f'Failed to get code with ID {id}: {response["msg"]}')
        elif 'login' in response.get('msg', ''):
            raise GenshinHelperException('Failed to get code: '
            'The cookie seems to be invalid, please re-login to m.weibo.cn')
        else:
            log.error(f'Failed to get code with ID {id}: {response}')

        return f'{id}: {response["msg"]}'

    def run(self):
        is_event = self.check_event(subscribe = True)
        if not is_event:
            message = '原神超话现在没有活动哦'
            return message

        ids = self.unclaimed_gift_ids
        if not ids:
            recent_codes = ' *'.join([f"{i['title']} {i['code']}" for i in self.gift_bag[:3]])
            message = f'原神超话签到活动已开启，但是没有未领取的兑换码。\n    最近 3 个码: {recent_codes}'
            return message

        log.info(f'检测到有 {len(ids)} 个未领取的兑换码')
        codes = [self.get_code(id) for id in ids]
        return '\n    '.join(codes)




from .basenotifier import BaseNotifier as Base
from ..config import config
from ..utils import log, request, _


class WechatWorkApp(Base):
    def __init__(self):
        self.name = 'Wechat Work App'
        self.token = config.WW_APP_AGENTID if self.access_token and config.WW_APP_AGENTID else ''
        self.retcode_key = 'errcode'
        self.retcode_value = 0

    @property
    def access_token(self):
        if config.WW_ID and config.WW_APP_SECRET:
            url = 'https://qyapi.weixin.qq.com/cgi-bin/gettoken'
            data = {'corpid': config.WW_ID, 'corpsecret': config.WW_APP_SECRET}

            try:
                response = request('get', url, params=data).json()
            except Exception as e:
                log.error(e)
            else:
                retcode = response.get('errcode')
                if retcode == 0:
                    return response['access_token']
                else:
                    log.error(_('Failed to get access_token:\n{response}'))
                    return
        else:
            return

    def send(self, text=Base.app, status=Base.status, desp=Base.desp):
        url = f'https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token={self.access_token}'
        data = {
            'touser': config.WW_APP_USERID,
            'msgtype': 'text',
            'agentid': config.WW_APP_AGENTID,
            'text': {
                'content': f'{text} {status}\n\n{desp}'
            }
        }
        return self.push('post', url, json=data)



from .basenotifier import BaseNotifier as Base
from ..config import config


class Bark(Base):
    def __init__(self):
        self.name = 'Bark App'
        self.token = config.BARK_KEY
        self.retcode_key = 'code'
        self.retcode_value = 200

    def send(self, text=Base.app, status=Base.status, desp=Base.desp):
        url = config.BARK_KEY
        data = {
            'title': f'{text} {status}',
            'body': desp,
            'sound': config.BARK_SOUND
        }
        return self.push('post', url, data=data)



from time import sleep
from random import randint

from . import notifiers
from ._version import __version__
from .config import config
from .genshin import YuanshenCheckin, GenshinCheckin
from .honkai3 import Bh3Checkin
from .miyoubi import MiyoubiCheckin
from .utils import log, get_cookies
from .weibo import SuperTopicCheckin, RedemptionCode
from .utils import _


banner = """
░█▀▀▀░█▀▀░█▀▀▄░█▀▀░█░░░░░▀░░█▀▀▄░█░░░░█▀▀░█░░▄▀▀▄░█▀▀░█▀▀▄
░█░▀▄░█▀▀░█░▒█░▀▀▄░█▀▀█░░█▀░█░▒█░█▀▀█░█▀▀░█░░█▄▄█░█▀▀░█▄▄▀
░▀▀▀▀░▀▀▀░▀░░▀░▀▀▀░▀░░▀░▀▀▀░▀░░▀░▀░░▀░▀▀▀░▀▀░█░░░░▀▀▀░▀░▀▀
"""
exit_code = 0
tasks = {
    'mihoyo': [
        '原神签到福利',
        get_cookies(config.COOKIE_MIHOYOBBS),
        YuanshenCheckin
    ],
    'bh3': [
        '崩坏3福利补给',
        get_cookies(config.COOKIE_BH3),
        Bh3Checkin
    ],
    'miyoubi': [
        '米游币签到姬',
        get_cookies(config.COOKIE_MIYOUBI),
        MiyoubiCheckin
    ],
    'hoyolab': [
        'HoYoLAB Community',
        get_cookies(config.COOKIE_HOYOLAB),
        GenshinCheckin
    ],
    'weibo': [
        '微博超话签到',
        get_cookies(config.COOKIE_WEIBO),
        SuperTopicCheckin
    ],
    'ka': [
        '原神超话监测',
        get_cookies(config.COOKIE_KA),
        RedemptionCode
    ]
}


def __run_sign(name, cookies, func):
    success_count = 0
    failure_count = 0
    if not cookies:
        return [success_count, failure_count]

    account_count = len(cookies)
    account_str = 'account' if account_count == 1 else 'accounts'
    log.info(_('You have {account_count} 「{name}」 {account_str} configured.').format(
        account_count=account_count, name=name, account_str=account_str
        ))
    global exit_code
    result_list = []
    for i, cookie in enumerate(cookies, start=1):
        log.info(_('Preparing to perform tasks for account {i}...').format(i=i))
        try:
            result = func(cookie).run()
            success_count += 1
        except Exception as e:
            result = e
            log.exception('TRACEBACK')
            failure_count += 1
            exit_code = -1
        finally:
            result = f'🌈 No.{i}:\n    {result}\n'
            result_list.append(result)
        continue

    message_box = [
        success_count,
        failure_count,
        f'🏆 {name}',
        f'☁️ ✔ {success_count} · ✖ {failure_count}',
        ''.join(result_list)
    ]
    return message_box


def main():
    log.info(banner)
    log.info(f'🌀 genshinhelper v{__version__}')

    if config.RUN_ENV == 'prod':
        sleep_secs = randint(10, int(config.MAX_SLEEP_SECS))
        log.info(_('Sleep for {} seconds...').format(sleep_secs))
        sleep(sleep_secs)

    log.info(_('Starting...'))

    result = {i[0]: __run_sign(i[1][0], i[1][1], i[1][2]) for i in tasks.items()}
    total_success = sum([i[0] for i in result.values()])
    total_failure = sum([i[1] for i in result.values()])
    message = sum([i[2::] for i in result.values()], [])
    tip = _('WARNING: Please configure environment variables or config.json file first!\n')
    message_box = '\n'.join(message) if message else tip

    log.info(_('RESULT:\n') + message_box)
    # The ``` is added to use markdown code block
    markdown_message = f'```\n{message_box}```'
    if message_box != tip:
        try:
            notifiers.send2all(
                status=f' ✔ {total_success} · ✖ {total_failure}', desp=markdown_message)
        except Exception as e:
            log.exception('TRACEBACK')

    if exit_code != 0:
        log.error(_('Process finished with exit code {exit_code}').format(exit_code=exit_code))
        exit(exit_code)
    log.info(_('End of process run'))

